@@ -1,27 +1,30 @@
-from datetime import datetime, date, time
-from decimal import Decimal
-import hashlib
-from operator import itemgetter
 from collections import OrderedDict
+from datetime import datetime, date, time, timedelta
+from decimal import Decimal
+from operator import itemgetter
+import hashlib
 
-from django import forms, VERSION
+from django import forms, VERSION, conf
 from django.apps import apps
-from django.conf.urls import url
 from django.contrib import admin, messages
 from django.contrib.admin import widgets
 from django.contrib.admin.options import csrf_protect_m
 from django.core.exceptions import PermissionDenied, ImproperlyConfigured
+from django.core.files.storage import default_storage
 from django.forms import fields
 from django.http import HttpResponseRedirect
 from django.template.response import TemplateResponse
-from django.utils import six
+from django.utils import timezone
 from django.utils.encoding import smart_bytes
 from django.utils.formats import localize
 from django.utils.module_loading import import_string
-from django.utils.translation import ugettext_lazy as _
-
+from django.utils.text import normalize_newlines
+from django.utils.translation import gettext_lazy as _
+from django.urls import path
 
 from . import LazyConfig, settings
+from .checks import get_inconsistent_fieldnames
+
 
 config = LazyConfig()
 
@@ -42,6 +45,9 @@ FIELDS = {
     datetime: (
         fields.SplitDateTimeField, {'widget': widgets.AdminSplitDateTime}
     ),
+    timedelta: (
+        fields.DurationField, {'widget': widgets.AdminTextInputWidget}
+    ),
     date: (fields.DateField, {'widget': widgets.AdminDateWidget}),
     time: (fields.TimeField, {'widget': widgets.AdminTimeWidget}),
     float: (fields.FloatField, {'widget': NUMERIC_WIDGET}),
@@ -73,12 +79,6 @@ def parse_additional_fields(fields):
 
 FIELDS.update(parse_additional_fields(settings.ADDITIONAL_FIELDS))
 
-if not six.PY3:
-    FIELDS.update({
-        long: INTEGER_LIKE,
-        unicode: STRING_LIKE,
-    })
-
 
 def get_values():
     """
@@ -90,8 +90,7 @@ def get_values():
     default_initial = ((name, options[0])
                        for name, options in settings.CONFIG.items())
     # Then update the mapping with actually values from the backend
-    initial = dict(default_initial,
-                   **dict(config._backend.mget(settings.CONFIG.keys())))
+    initial = dict(default_initial, **dict(config._backend.mget(settings.CONFIG)))
 
     return initial
 
@@ -99,15 +98,22 @@ def get_values():
 class ConstanceForm(forms.Form):
     version = forms.CharField(widget=forms.HiddenInput)
 
-    def __init__(self, initial, *args, **kwargs):
-        super(ConstanceForm, self).__init__(*args, initial=initial, **kwargs)
-        version_hash = hashlib.md5()
+    def __init__(self, initial, request=None, *args, **kwargs):
+        super().__init__(*args, initial=initial, **kwargs)
+        version_hash = hashlib.sha256()
+
+        only_view = request and not request.user.has_perm('constance.change_config')
+        if only_view:
+            messages.warning(
+                request,
+                _("You don't have permission to change these values"),
+            )
 
         for name, options in settings.CONFIG.items():
-            default, help_text = options[0], options[1]
+            default = options[0]
             if len(options) == 3:
                 config_type = options[2]
-                if config_type not in settings.ADDITIONAL_FIELDS and not isinstance(options[0], config_type):
+                if config_type not in settings.ADDITIONAL_FIELDS and not isinstance(default, config_type):
                     raise ImproperlyConfigured(_("Default value type must be "
                                                  "equal to declared config "
                                                  "parameter type. Please fix "
@@ -125,15 +131,30 @@ class ConstanceForm(forms.Form):
                                            % {'config_type': config_type,
                                               'name': name})
             field_class, kwargs = FIELDS[config_type]
+            if only_view:
+                kwargs['disabled'] = True
             self.fields[name] = field_class(label=name, **kwargs)
 
             version_hash.update(smart_bytes(initial.get(name, '')))
         self.initial['version'] = version_hash.hexdigest()
 
     def save(self):
+        for file_field in self.files:
+            file = self.cleaned_data[file_field]
+            self.cleaned_data[file_field] = default_storage.save(file.name, file)
+
         for name in settings.CONFIG:
-            if getattr(config, name) != self.cleaned_data[name]:
-                setattr(config, name, self.cleaned_data[name])
+            current = getattr(config, name)
+            new = self.cleaned_data[name]
+
+            if isinstance(new, str):
+                new = normalize_newlines(new)
+
+            if conf.settings.USE_TZ and isinstance(current, datetime) and not timezone.is_aware(current):
+                current = timezone.make_aware(current)
+
+            if current != new:
+                setattr(config, name, new)
 
     def clean_version(self):
         value = self.cleaned_data['version']
@@ -147,6 +168,18 @@ class ConstanceForm(forms.Form):
                                           'form and resubmit your changes.'))
         return value
 
+    def clean(self):
+        cleaned_data = super().clean()
+
+        if not settings.CONFIG_FIELDSETS:
+            return cleaned_data
+
+        if get_inconsistent_fieldnames():
+            raise forms.ValidationError(_('CONSTANCE_CONFIG_FIELDSETS is missing '
+                                          'field(s) that exists in CONSTANCE_CONFIG.'))
+
+        return cleaned_data
+
 
 class ConstanceAdmin(admin.ModelAdmin):
     change_list_template = 'admin/constance/change_list.html'
@@ -155,10 +188,10 @@ class ConstanceAdmin(admin.ModelAdmin):
     def get_urls(self):
         info = self.model._meta.app_label, self.model._meta.module_name
         return [
-            url(r'^$',
+            path('',
                 self.admin_site.admin_view(self.changelist_view),
                 name='%s_%s_changelist' % info),
-            url(r'^$',
+            path('',
                 self.admin_site.admin_view(self.changelist_view),
                 name='%s_%s_add' % info),
         ]
@@ -178,8 +211,10 @@ class ConstanceAdmin(admin.ModelAdmin):
             'value': localize(value),
             'modified': localize(value) != localize(default),
             'form_field': form[name],
-            'is_checkbox': isinstance(
-                form[name].field.widget, forms.CheckboxInput),
+            'is_date': isinstance(default, date),
+            'is_datetime': isinstance(default, datetime),
+            'is_checkbox': isinstance(form[name].field.widget, forms.CheckboxInput),
+            'is_file': isinstance(form[name].field.widget, forms.FileInput),
         }
 
         return config_value
@@ -194,13 +229,15 @@ class ConstanceAdmin(admin.ModelAdmin):
 
     @csrf_protect_m
     def changelist_view(self, request, extra_context=None):
-        if not self.has_change_permission(request, None):
+        if not self.has_view_or_change_permission(request):
             raise PermissionDenied
         initial = get_values()
         form_cls = self.get_changelist_form(request)
-        form = form_cls(initial=initial)
-        if request.method == 'POST':
-            form = form_cls(data=request.POST, initial=initial)
+        form = form_cls(initial=initial, request=request)
+        if request.method == 'POST' and request.user.has_perm('constance.change_config'):
+            form = form_cls(
+                data=request.POST, files=request.FILES, initial=initial
+            )
             if form.is_valid():
                 form.save()
                 messages.add_message(
@@ -210,7 +247,7 @@ class ConstanceAdmin(admin.ModelAdmin):
                 )
                 return HttpResponseRedirect('.')
         context = dict(
-            admin.site.each_context(request),
+            self.admin_site.each_context(request),
             config_values=[],
             title=self.model._meta.app_config.verbose_name,
             app_label='constance',
@@ -226,23 +263,36 @@ class ConstanceAdmin(admin.ModelAdmin):
 
         if settings.CONFIG_FIELDSETS:
             context['fieldsets'] = []
-            for fieldset_title, fields_list in settings.CONFIG_FIELDSETS.items():
-                fields_exist = all(
-                    field in settings.CONFIG.keys() for field in fields_list
-                )
-                assert fields_exist, "CONSTANCE_CONFIG_FIELDSETS contains fields that does not exist"
+            for fieldset_title, fieldset_data in settings.CONFIG_FIELDSETS.items():
+                if type(fieldset_data) == dict:
+                    fields_list = fieldset_data['fields']
+                    collapse = fieldset_data.get('collapse', False)
+                else:
+                    fields_list = fieldset_data
+                    collapse = False
+
+                absent_fields = [field for field in fields_list
+                                 if field not in settings.CONFIG]
+                assert not any(absent_fields), (
+                    "CONSTANCE_CONFIG_FIELDSETS contains field(s) that does "
+                    "not exist: %s" % ', '.join(absent_fields))
+
                 config_values = []
 
-                for name, options in settings.CONFIG.items():
-                    if name in fields_list:
+                for name in fields_list:
+                    options = settings.CONFIG.get(name)
+                    if options:
                         config_values.append(
                             self.get_config_value(name, options, form, initial)
                         )
-
-                context['fieldsets'].append({
+                fieldset_context = {
                     'title': fieldset_title,
                     'config_values': config_values
-                })
+                }
+
+                if collapse:
+                    fieldset_context['collapse'] = True
+                context['fieldsets'].append(fieldset_context)
             if not isinstance(settings.CONFIG_FIELDSETS, OrderedDict):
                 context['fieldsets'].sort(key=itemgetter('title'))
 
@@ -260,11 +310,11 @@ class ConstanceAdmin(admin.ModelAdmin):
     def has_change_permission(self, request, obj=None):
         if settings.SUPERUSER_ONLY:
             return request.user.is_superuser
-        return super(ConstanceAdmin, self).has_change_permission(request, obj)
+        return super().has_change_permission(request, obj)
 
 
-class Config(object):
-    class Meta(object):
+class Config:
+    class Meta:
         app_label = 'constance'
         object_name = 'Config'
         model_name = module_name = 'config'
@@ -282,6 +332,14 @@ class Config(object):
         def app_config(self):
             return apps.get_app_config(self.app_label)
 
+        @property
+        def label(self):
+            return '%s.%s' % (self.app_label, self.object_name)
+
+        @property
+        def label_lower(self):
+            return '%s.%s' % (self.app_label, self.model_name)
+
     _meta = Meta()
 
 
