@@ -1,6 +1,6 @@
 from django.db.models import signals
 from django.apps import AppConfig
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 
 class ConstanceConfig(AppConfig):
@@ -8,7 +8,7 @@ class ConstanceConfig(AppConfig):
     verbose_name = _('Constance')
 
     def ready(self):
-        super(ConstanceConfig, self).ready()
+        super().ready()
         signals.post_migrate.connect(self.create_perm,
                                      dispatch_uid='constance.create_perm')
 
@@ -17,16 +17,26 @@ class ConstanceConfig(AppConfig):
         Creates a fake content type and permission
         to be able to check for permissions
         """
+        from django.conf import settings
         from django.contrib.auth.models import Permission
         from django.contrib.contenttypes.models import ContentType
 
+        constance_dbs = getattr(settings, 'CONSTANCE_DBS', None)
+        if constance_dbs is not None and using not in constance_dbs:
+            return
         if ContentType._meta.installed and Permission._meta.installed:
             content_type, created = ContentType.objects.using(using).get_or_create(
                 app_label='constance',
                 model='config',
             )
 
-            permission, created = Permission.objects.using(using).get_or_create(
-                name='Can change config',
+            Permission.objects.using(using).get_or_create(
                 content_type=content_type,
-                codename='change_config')
+                codename='change_config',
+                defaults={'name': 'Can change config'},
+            )
+            Permission.objects.using(using).get_or_create(
+                content_type=content_type,
+                codename='view_config',
+                defaults={'name': 'Can view config'},
+            )
