@@ -1,6 +1,12 @@
 from django.core.cache import caches
 from django.core.cache.backends.locmem import LocMemCache
 from django.core.exceptions import ImproperlyConfigured
+from django.db import (
+    IntegrityError,
+    OperationalError,
+    ProgrammingError,
+    transaction,
+)
 from django.db.models.signals import post_save
 
 from .. import Backend
@@ -45,17 +51,20 @@ class DatabaseBackend(Backend):
             return
         autofill_values = {}
         autofill_values[full_cachekey] = 1
-        for key, value in self.mget(settings.CONFIG.keys()):
+        for key, value in self.mget(settings.CONFIG):
             autofill_values[self.add_prefix(key)] = value
         self._cache.set_many(autofill_values, timeout=self._autofill_timeout)
 
     def mget(self, keys):
         if not keys:
             return
-        keys = dict((self.add_prefix(key), key) for key in keys)
-        stored = self._model._default_manager.filter(key__in=keys.keys())
-        for const in stored:
-            yield keys[const.key], const.value
+        keys = {self.add_prefix(key): key for key in keys}
+        try:
+            stored = self._model._default_manager.filter(key__in=keys)
+            for const in stored:
+                yield keys[const.key], const.value
+        except (OperationalError, ProgrammingError):
+            pass
 
     def get(self, key):
         key = self.add_prefix(key)
@@ -69,7 +78,7 @@ class DatabaseBackend(Backend):
         if value is None:
             try:
                 value = self._model._default_manager.get(key=key).value
-            except self._model.DoesNotExist:
+            except (OperationalError, ProgrammingError, self._model.DoesNotExist):
                 pass
             else:
                 if self._cache:
@@ -77,13 +86,34 @@ class DatabaseBackend(Backend):
         return value
 
     def set(self, key, value):
-        old_value = self.get(key)
-        constance, created = self._model._default_manager.get_or_create(
-            key=self.add_prefix(key), defaults={'value': value}
-        )
+        key = self.add_prefix(key)
+        created = False
+        queryset = self._model._default_manager.all()
+        # Set _for_write attribute as get_or_create method does
+        # https://github.com/django/django/blob/2.2.11/django/db/models/query.py#L536
+        queryset._for_write = True
+
+        try:
+            constance = queryset.get(key=key)
+        except (OperationalError, ProgrammingError):
+            # database is not created, noop
+            return
+        except self._model.DoesNotExist:
+            try:
+                with transaction.atomic(using=queryset.db):
+                    queryset.create(key=key, value=value)
+                created = True
+            except IntegrityError as error:
+                # Allow concurrent writes
+                constance = queryset.get(key=key)
+
         if not created:
+            old_value = constance.value
             constance.value = value
             constance.save()
+        else:
+            old_value = None
+
         if self._cache:
             self._cache.set(key, value)
 
@@ -93,7 +123,7 @@ class DatabaseBackend(Backend):
 
     def clear(self, sender, instance, created, **kwargs):
         if self._cache and not created:
-            keys = [self.add_prefix(k) for k in settings.CONFIG.keys()]
+            keys = [self.add_prefix(k) for k in settings.CONFIG]
             keys.append(self.add_prefix(self._autofill_cachekey))
             self._cache.delete_many(keys)
             self.autofill()
